about summary refs log tree commit diff
path: root/bits/libc-tsd.h
diff options
context:
space:
mode:
authorRoland McGrath <roland@gnu.org>2002-08-02 03:32:24 +0000
committerRoland McGrath <roland@gnu.org>2002-08-02 03:32:24 +0000
commitc2afe833521105e05298500ba5d4676d6c833242 (patch)
tree226bb6da849fe8d860b766d3c365c8cfeb99b19e /bits/libc-tsd.h
parentae8f7f19faf701667d5e88195e3efc479c4a0746 (diff)
downloadglibc-c2afe833521105e05298500ba5d4676d6c833242.tar.gz
glibc-c2afe833521105e05298500ba5d4676d6c833242.tar.xz
glibc-c2afe833521105e05298500ba5d4676d6c833242.zip
* sysdeps/pthread/bits/libc-tsd.h: Include <tls.h>.
	[USE_TLS && HAVE___THREAD]: Just include the sysdeps/generic file,
	which does the right thing when __thread support is available.
	* descr.h (struct _pthread_descr_struct) [USE_TLS && HAVE___THREAD]:
	Omit `p_libc_specific', `p_errnop', `p_errno', `p_h_errnop',
	`p_h_errno', `p_resp', and `p_res' members.
	* pthread.c (__pthread_initialize_minimal) [USE_TLS && HAVE___THREAD]:
	Don't initialize `p_errnop' and `p_h_errnop' members.
	(__pthread_reset_main_thread): Likewise.
	(__pthread_initialize_manager): Likewise.
	* manager.c (__pthread_manager, pthread_handle_create): Likewise.
	* pthread.c (pthread_initialize) [USE_TLS && HAVE___THREAD]:
	Don't initialize `p_resp' member.
	(__pthread_reset_main_thread): Likewise.
	* manager.c (pthread_handle_create): Likewise.
	* specific.c (libc_internal_tsd_set, libc_internal_tsd_get):
	Conditionalize these on [!(USE_TLS && HAVE___THREAD)].
	* errno.c [USE_TLS && HAVE___THREAD]
	(__h_errno_location, __res_state): Don't define these at all.

	* sysdeps/i386/tls.h (INSTALL_DTV): Add parens around arguments!
	(INSTALL_NEW_DTV, GET_DTV): Likewise.
	* sysdeps/sh/tls.h (INSTALL_DTV, INSTALL_NEW_DTV, GET_DTV): Likewise.

	* weaks.c: Don't include <errno.h> here.

2002-08-01  Roland McGrath  <roland@redhat.com>

	* sysdeps/i386/tls.h (TLS_DO_MODIFY_LDT): New macro, broken out of
	TLS_INIT_TP.
	(TLS_DO_SET_THREAD_AREA): New macro, uses thread_set_area syscall.
	(TLS_SETUP_GS_SEGMENT): New macro, try one or the other or both.
	(TLS_INIT_TP): Use that.
Diffstat (limited to 'bits/libc-tsd.h')
-rw-r--r--bits/libc-tsd.h29
1 files changed, 19 insertions, 10 deletions
diff --git a/bits/libc-tsd.h b/bits/libc-tsd.h
index 80bd1d33dc..fc17be108e 100644
--- a/bits/libc-tsd.h
+++ b/bits/libc-tsd.h
@@ -1,5 +1,5 @@
-/* libc-internal interface for thread-specific data.  Stub version.
-   Copyright (C) 1998, 2001 Free Software Foundation, Inc.
+/* libc-internal interface for thread-specific data.  Stub or TLS version.
+   Copyright (C) 1998,2001,02 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -17,8 +17,8 @@
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA.  */
 
-#ifndef _BITS_LIBC_TSD_H
-#define _BITS_LIBC_TSD_H 1
+#ifndef _GENERIC_BITS_LIBC_TSD_H
+#define _GENERIC_BITS_LIBC_TSD_H 1
 
 /* This file defines the following macros for accessing a small fixed
    set of thread-specific `void *' data used only internally by libc.
@@ -40,13 +40,22 @@
    Some implementations may not provide any enum at all and instead
    using string pasting in the macros.  */
 
-/* This is the generic/stub implementation for wholly single-threaded
-   systems.  We don't define an enum for the possible key values, because
-   the KEYs translate directly into variables by macro magic.  */
+#include <tls.h>
 
-#define __libc_tsd_define(CLASS, KEY)	CLASS void *__libc_tsd_##KEY##_data;
-#define __libc_tsd_get(KEY)		(__libc_tsd_##KEY##_data)
-#define __libc_tsd_set(KEY, VALUE)	(__libc_tsd_##KEY##_data = (VALUE))
+/* When full support for __thread variables is available, this interface is
+   just a trivial wrapper for it.  Without TLS, this is the generic/stub
+   implementation for wholly single-threaded systems.
 
+   We don't define an enum for the possible key values, because the KEYs
+   translate directly into variables by macro magic.  */
+
+#if USE_TLS && HAVE___THREAD
+# define __libc_tsd_define(CLASS, KEY)	CLASS __thread void *__libc_tsd_##KEY;
+#else
+# define __libc_tsd_define(CLASS, KEY)	CLASS void *__libc_tsd_##KEY;
+#endif
+
+#define __libc_tsd_get(KEY)		(__libc_tsd_##KEY)
+#define __libc_tsd_set(KEY, VALUE)	(__libc_tsd_##KEY = (VALUE))
 
 #endif	/* bits/libc-tsd.h */